/*
 * Copyright 2012-2013, Haiku, Inc. All Rights Reserved.
 * Distributed under the terms of the MIT License.
 *
 * Authors:
 *	Alexander von Gluck IV, kallisti5@unixzen.com
 */


#include "accelerant_protos.h"
#include "accelerant.h"

#include <string.h>


status_t
intel_pm_downclock(bool enable)
{
	if (!gInfo->shared_info->device_type.InGroup(GEN6_TYPE_SNB)
		&& !gInfo->shared_info->device_type.InGroup(GEN6_TYPE_IVB)) {
		TRACE("%s: Downclocking not supported on this chipset.\n", __func__);
		return B_ERROR;
	}

	// TODO: Check for deep RC6
	// IvyBridge, SandyBridge, and Haswell can do depth 1 atm
	// Some chipsets can go deeper... but this is safe for now
	// Haswell should *NOT* do over depth 1;
	int depth = 1;

	/* Magical sequence of register writes to enable
	 * downclocking from the fine folks at Xorg
	 */
	write32(GEN6_RC_STATE, 0);
	// TODO: We may need grab rps states here
	write32(GEN6_RC_CONTROL, 0);

	write32(GEN6_RC1_WAKE_RATE_LIMIT, 1000 << 16);
	write32(GEN6_RC6_WAKE_RATE_LIMIT, 40 << 16 | 30);
	write32(GEN6_RC6pp_WAKE_RATE_LIMIT, 30);
	write32(GEN6_RC_EVALUATION_INTERVAL, 125000);
	write32(GEN6_RC_IDLE_HYSTERSIS, 25);

	// TODO: Idle each ring

	write32(GEN6_RC_SLEEP, 0);
	write32(GEN6_RC1e_THRESHOLD, 1000);
	write32(GEN6_RC6_THRESHOLD, 50000);
	write32(GEN6_RC6p_THRESHOLD, 100000);
	write32(GEN6_RC6pp_THRESHOLD, 64000);

	uint32 rc6Mask = GEN6_RC_CTL_RC6_ENABLE;

	if (depth > 1)
		rc6Mask |= GEN6_RC_CTL_RC6p_ENABLE;
	if (depth > 2)
		rc6Mask |= GEN6_RC_CTL_RC6pp_ENABLE;

	write32(GEN6_RC_CONTROL, rc6Mask | 
	write32(GEN6_RPNSWREQ,
	write32(GEN6_RC_VIDEO_FREQ, 

	write32(GEN6_RP_DOWN_TIMEOUT, 1000000);
	write32(GEN6_RP_INTERRUPT_LIMITS,
		maxDelay << 24 | minDelay << 16);

	write32(GEN6_RP_UP_THRESHOLD, 59400);
	write32(GEN6_RP_DOWN_THRESHOLD, 245000);
	write32(GEN6_RP_UP_EI, 66000);
	write32(GEN6_RP_DOWN_EI, 350000);

	write32(GEN6_RP_IDLE_HYSTERSIS, 10);
	write32(GEN6_RP_CONTROL, GEN6_RP_MEDIA_TURBO
		| GEN6_RP_MEDIA_HW_NORMAL_MODE | GEN6_RP_MEDIA_IS_GFX
		| GEN6_RP_ENABLE | GEN6_RP_UP_BUSY_AVG
		| GEN6_RP_DOWN_IDLE_CONT);
		// TODO: | (HASWELL ? GEN7_RP_DOWN_IDLE_AVG : GEN6_RP_DOWN_IDLE_CONT));

	// TODO: wait for (read32(GEN6_PCODE_MAILBOX) & GEN6_PCODE_READY)

	write32(GEN6_PCODE_DATA, 0);
	write32(GEN6_PCODE_MAILBOX, GEN6_PCODE_READY
		| GEN6_PCODE_WRITE_MIN_FREQ_TABLE);

	// TODO: wait for (read32(GEN6_PCODE_MAILBOX) & GEN6_PCODE_READY)

	// TODO: check for overclock support and set.	

}
